'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = bindImports;

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function detective(input) {
  const dependencies = [];
  const ast = (0, _lodash2.default)({}, input);

  (0, _babelTraverse2.default)(ast, {
    VariableDeclarator(path) {
      const init = path.node.init;
      const identifier = path.node.id;
      const isCallExpression = t.isCallExpression(init);

      const isRequireCall = isCallExpression && init.callee.name === 'require';

      const isStaticRequireCall = isRequireCall && init.arguments.length === 1 && t.isStringLiteral(init.arguments[0]);

      // like: var foo = require('./bar');
      if (isStaticRequireCall) {
        const sourceName = init.arguments[0].value;
        // Only work on local requires
        if (sourceName[0] !== '.') {
          return;
        }

        dependencies.push({
          specifierName: identifier.name,
          sourceName: sourceName,
          path: path,
          type: 'default'
        });
      }
    }
  });

  return dependencies;
}

function bindImports(input) {
  const ast = (0, _lodash2.default)({}, input);
  const dependencies = detective(ast);

  (0, _babelTraverse2.default)(ast, {
    Program(path) {
      const containerPath = path.scope.generateUidIdentifier('modules');

      const properties = dependencies.map(dependency => {
        const { specifierName } = dependency;
        const init = dependency.path.node.init;

        dependency.path.remove();
        dependency.path.scope.rename(specifierName, `${ containerPath.name }.${ specifierName }`);

        return t.objectMethod('get', t.identifier(specifierName), [], t.BlockStatement([t.returnStatement(init)]));
      });

      if (properties.length > 0) {
        const modules = t.objectExpression(properties);
        const modulesDeclaration = t.VariableDeclaration('const', [t.VariableDeclarator(containerPath, modules)]);
        path.node.body.unshift(modulesDeclaration);
      }
    }
  });

  return ast;
}
module.exports = exports['default'];