'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = configure;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _vm = require('vm');

var _vm2 = _interopRequireDefault(_vm);

var _events = require('events');

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _babylon = require('babylon');

var _babelCore = require('babel-core');

var _babelCodeFrame = require('babel-code-frame');

var _babelCodeFrame2 = _interopRequireDefault(_babelCodeFrame);

var _bindImports = require('./bind-imports');

var _bindImports2 = _interopRequireDefault(_bindImports);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let called = false;
function configure(entrypoint, overrideRequires = {}, opts = {}) {
  if (called) throw new Error('You can only use babel-live once per project');
  called = true;
  const emitter = new _events.EventEmitter();

  opts = opts || {};
  // if (opts.sourceMap !== false) opts.sourceMap = "inline";

  let requireInProgress = false;
  let extraRequireNeeded = false;
  let requireCache = {};
  // filename => fn(module, exports, require, __filename, __dirname)
  const moduleCache = {};
  function invalidate(filename) {
    requireCache = {};
    moduleCache[filename] = null;
    doRequire();
  }
  function doRequire() {
    if (!requireInProgress) {
      requireInProgress = true;
      try {
        babelRequire(entrypoint);
        emitter.emit('hotswap');
      } catch (error) {
        emitter.emit('error', error);
      }
      setTimeout(() => {
        requireInProgress = false;
        if (extraRequireNeeded) {
          extraRequireNeeded = false;
          doRequire();
        }
      }, 2000);
    } else {
      extraRequireNeeded = true;
    }
  }

  const watching = {};
  function watch(filename) {
    if (watching[filename]) return;
    watching[filename] = true;
    const w = _chokidar2.default.watch(filename, {
      persistent: true,
      usePolling: true,
      interval: 100
    });
    w.on('error', err => {
      console.log('error watching file');
      console.log('probably nothing to worry about');
      console.log(err.message);
    });
    w.on('change', () => invalidate(filename));
  }

  function babelRequire(filename) {
    filename = _path2.default.resolve(filename);
    watch(filename);
    if (requireCache[filename]) return requireCache[filename];
    let fn = moduleCache[filename];
    if (!fn) {
      const src = babelLoad(filename);
      fn = _vm2.default.runInThisContext('(function(module,exports,require,__filename,__dirname){' + src + '\n})', filename);
      moduleCache[filename] = fn;
    }
    requireCache[filename] = {};
    const mod = {
      exports: requireCache[filename],
      hotswap: emitter
    };
    function proxiedRequire(id) {
      if (overrideRequires[id]) {
        return overrideRequires[id];
      }
      const p = _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json']
      });
      if (/^\./.test(id)) {
        return babelRequire(p);
      } else {
        return require(p);
      }
    }
    proxiedRequire.resolve = id => {
      return _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json']
      });
    };
    const sandbox = {
      'module': mod,
      'exports': mod.exports,
      'require': proxiedRequire,
      '__filename': filename,
      '__dirname': _path2.default.dirname(filename)
    };
    fn(sandbox.module, sandbox.exports, sandbox.require, sandbox.__filename, sandbox.__dirname);
    return requireCache[filename] = mod.exports;
  }

  function babelLoad(filename) {
    const src = _fs2.default.readFileSync(filename, 'utf8');

    try {
      const ast = (0, _babylon.parse)(src, {
        sourceType: 'module',
        plugins: ['jsx', 'asyncFunctions', 'classConstructorCall', 'doExpressions', 'trailingFunctionCommas', 'objectRestSpread', 'decorators', 'classProperties', 'exportExtensions', 'exponentiationOperator', 'asyncGenerators', 'functionBind', 'functionSent']
      });

      const pretransformed = (0, _babelCore.transformFromAst)(ast, src, {
        filename: filename,
        presets: [],
        plugins: ['transform-es2015-modules-commonjs']
      });

      const boundmodules = (0, _bindImports2.default)(pretransformed.ast);

      opts.filename = filename;
      const { code } = (0, _babelCore.transformFromAst)(boundmodules, src, opts);
      return code;
    } catch (error) {
      const { loc } = error;

      if (loc) {
        const frame = (0, _babelCodeFrame2.default)(src, loc.line, loc.column + 1, opts);
        error.codeFrame = frame;
        error.message = [error.message, frame].join('\n');
      }
      throw error;
    }
  }

  return babelRequire(entrypoint);
}
module.exports = exports['default'];